from gradio.components.base import Component


class mcp_tools_visualizer(Component):

    EVENTS = [Events.input]

    def __init__(
        self,
        value: str | Callable | None = None,
        *,
        label: str | I18nData | None = None,
        every: Timer | float | None = None,
        inputs: Component | Sequence[Component] | set[Component] | None = None,
        show_label: bool = False,
        visible: bool = True,
        elem_id: str | None = None,
        elem_classes: list[str] | str | None = None,
        render: bool = True,
        key: int | str | tuple[int | str, ...] | None = None,
        preserved_by_key: list[str] | str | None = "value",
        min_height: int | None = None,
        max_height: int | None = None,
        container: bool = False,
        padding: bool = True,
    ):
        self.min_height = min_height
        self.max_height = max_height
        self.padding = padding
        super().__init__(
            label=label,
            every=every,
            inputs=inputs,
            show_label=show_label,
            visible=visible,
            elem_id=elem_id,
            elem_classes=elem_classes,
            render=render,
            key=key,
            preserved_by_key=preserved_by_key,
            value=value,
            container=container,
        )

    def preprocess(self, payload):
        """
        Payload from frontend: { "action": "call_tool", "tool_name": str, "inputs": dict, "server_parameters": dict }
        """
        return {payload}

    def postprocess(self, value):
        """
        Frontend will receive initial data: { server_info, tool_summaries }
        """
        server_parameters = value.get("server_parameters")
        mcp_client = MCPClient(server_parameters)

        tool_summaries = self.get_tool_summaries(mcp_client.get_tools())
        return {
            "server_parameters": server_parameters,
            # "server_info": server_info,
            "tool_summaries": tool_summaries
        }

    def example_payload(self):
        return {
            "action": "call_tool",
            "tool_name": "example_tool",
            "inputs": {"param": "value"},
            "server_parameters": {"url": "http://huggingface.co/mcp/", "transport": "streamable-http"},
        }

    def example_value(self):
        return {
            "server_parameters": {"url": "http://huggingface.co/mcp/", "transport": "streamable-http"},
            "server_info": {},
            "tool_summaries": []
        }

    def api_info(self):
        return {"type": {}, "description": "any valid json"}

    def example_input_for_param(self, param_schema):
            param_type = param_schema.get("type", "string")
            if param_type == "string":
                return "example text"
            elif param_type == "integer":
                return 42
            elif param_type == "number":
                return 3.14
            elif param_type == "boolean":
                return True
            elif param_type == "array":
                return []
            elif param_type == "object":
                return {}
            else:
                return None

    def get_tool_summaries(self, tools):
            tool_summaries = []
            for tool in tools:
                example_inputs = {
                    param_name: self.example_input_for_param(param_schema)
                    for param_name, param_schema in tool.inputs.items()
                }
                tool_summary = {
                    "name": tool.name,
                    "description": tool.description,
                    "inputs": tool.inputs,
                    "example_inputs": example_inputs,
                    "output_type": tool.output_type,
                }
                tool_summaries.append(tool_summary)
            return tool_summaries
    from typing import Callable, Literal, Sequence, Any, TYPE_CHECKING
    from gradio.blocks import Block
    if TYPE_CHECKING:
        from gradio.components import Timer
        from gradio.components.base import Component

    
    def input(self,
        fn: Callable[..., Any] | None = None,
        inputs: Block | Sequence[Block] | set[Block] | None = None,
        outputs: Block | Sequence[Block] | None = None,
        api_name: str | None | Literal[False] = None,
        scroll_to_output: bool = False,
        show_progress: Literal["full", "minimal", "hidden"] = "full",
        show_progress_on: Component | Sequence[Component] | None = None,
        queue: bool | None = None,
        batch: bool = False,
        max_batch_size: int = 4,
        preprocess: bool = True,
        postprocess: bool = True,
        cancels: dict[str, Any] | list[dict[str, Any]] | None = None,
        every: Timer | float | None = None,
        trigger_mode: Literal["once", "multiple", "always_last"] | None = None,
        js: str | Literal[True] | None = None,
        concurrency_limit: int | None | Literal["default"] = "default",
        concurrency_id: str | None = None,
        show_api: bool = True,
        key: int | str | tuple[int | str, ...] | None = None,
    
        ) -> Dependency:
        """
        Parameters:
            fn: the function to call when this event is triggered. Often a machine learning model's prediction function. Each parameter of the function corresponds to one input component, and the function should return a single value or a tuple of values, with each element in the tuple corresponding to one output component.
            inputs: list of gradio.components to use as inputs. If the function takes no inputs, this should be an empty list.
            outputs: list of gradio.components to use as outputs. If the function returns no outputs, this should be an empty list.
            api_name: defines how the endpoint appears in the API docs. Can be a string, None, or False. If False, the endpoint will not be exposed in the api docs. If set to None, will use the functions name as the endpoint route. If set to a string, the endpoint will be exposed in the api docs with the given name.
            scroll_to_output: if True, will scroll to output component on completion
            show_progress: how to show the progress animation while event is running: "full" shows a spinner which covers the output component area as well as a runtime display in the upper right corner, "minimal" only shows the runtime display, "hidden" shows no progress animation at all
            show_progress_on: Component or list of components to show the progress animation on. If None, will show the progress animation on all of the output components.
            queue: if True, will place the request on the queue, if the queue has been enabled. If False, will not put this event on the queue, even if the queue has been enabled. If None, will use the queue setting of the gradio app.
            batch: if True, then the function should process a batch of inputs, meaning that it should accept a list of input values for each parameter. The lists should be of equal length (and be up to length `max_batch_size`). The function is then *required* to return a tuple of lists (even if there is only 1 output component), with each list in the tuple corresponding to one output component.
            max_batch_size: maximum number of inputs to batch together if this is called from the queue (only relevant if batch=True)
            preprocess: if False, will not run preprocessing of component data before running 'fn' (e.g. leaving it as a base64 string if this method is called with the `Image` component).
            postprocess: if False, will not run postprocessing of component data before returning 'fn' output to the browser.
            cancels: a list of other events to cancel when this listener is triggered. For example, setting cancels=[click_event] will cancel the click_event, where click_event is the return value of another components .click method. Functions that have not yet run (or generators that are iterating) will be cancelled, but functions that are currently running will be allowed to finish.
            every: continously calls `value` to recalculate it if `value` is a function (has no effect otherwise). Can provide a Timer whose tick resets `value`, or a float that provides the regular interval for the reset Timer.
            trigger_mode: if "once" (default for all events except `.change()`) would not allow any submissions while an event is pending. If set to "multiple", unlimited submissions are allowed while pending, and "always_last" (default for `.change()` and `.key_up()` events) would allow a second submission after the pending event is complete.
            js: optional frontend js method to run before running 'fn'. Input arguments for js method are values of 'inputs' and 'outputs', return should be a list of values for output components.
            concurrency_limit: if set, this is the maximum number of this event that can be running simultaneously. Can be set to None to mean no concurrency_limit (any number of this event can be running simultaneously). Set to "default" to use the default concurrency limit (defined by the `default_concurrency_limit` parameter in `Blocks.queue()`, which itself is 1 by default).
            concurrency_id: if set, this is the id of the concurrency group. Events with the same concurrency_id will be limited by the lowest set concurrency_limit.
            show_api: whether to show this event in the "view API" page of the Gradio app, or in the ".view_api()" method of the Gradio clients. Unlike setting api_name to False, setting show_api to False will still allow downstream apps as well as the Clients to use this event. If fn is None, show_api will automatically be set to False.
            key: A unique key for this event listener to be used in @gr.render(). If set, this value identifies an event as identical across re-renders when the key is identical.
        
        """
        ...



import gradio as gr
from gradio_mcp_tools_visualizer import mcp_tools_visualizer
from smolagents import MCPClient

server_parameters = {"url": "http://huggingface.co/mcp/", "transport": "streamable-http"}
with gr.Blocks() as demo:
    gr.Markdown("# MCP Tools Visualizer 🚀")
    mcp_tools_visualizer(
        value={
            'server_parameters': server_parameters
        },
    )

if __name__ == "__main__":
    demo.launch()



# import gradio as gr
# from gradio_mcp_tools_visualizer import mcp_tools_visualizer


# with gr.Blocks() as demo:
#     gr.Markdown("# Change the value (keep it JSON) and the front-end will update automatically.")
#     mcp_tools_visualizer(value={"message": "Hello from Gradio!"}, label="Static")


# if __name__ == "__main__":
#     demo.launch()


import json
import time

from collections.abc import Callable, Sequence
from typing import TYPE_CHECKING, Any

from gradio.components.base import Component
from gradio.components import Timer
from gradio.events import Events
from gradio.i18n import I18nData

from smolagents import MCPClient

# if TYPE_CHECKING:
#     print('type')
#     from gradio.components import Timer


class mcp_tools_visualizer(Component):

    EVENTS = [Events.input]

    def __init__(
        self,
        value: str | Callable | None = None,
        *,
        label: str | I18nData | None = None,
        every: Timer | float | None = None,
        inputs: Component | Sequence[Component] | set[Component] | None = None,
        show_label: bool = False,
        visible: bool = True,
        elem_id: str | None = None,
        elem_classes: list[str] | str | None = None,
        render: bool = True,
        key: int | str | tuple[int | str, ...] | None = None,
        preserved_by_key: list[str] | str | None = "value",
        min_height: int | None = None,
        max_height: int | None = None,
        container: bool = False,
        padding: bool = True,
    ):
        self.min_height = min_height
        self.max_height = max_height
        self.padding = padding
        super().__init__(
            label=label,
            every=every,
            inputs=inputs,
            show_label=show_label,
            visible=visible,
            elem_id=elem_id,
            elem_classes=elem_classes,
            render=render,
            key=key,
            preserved_by_key=preserved_by_key,
            value=value,
            container=container,
        )

    def preprocess(self, payload):
        """
        Payload from frontend: { "action": "call_tool", "tool_name": str, "inputs": dict, "server_parameters": dict }
        """
        return {payload}

    def postprocess(self, value):
        """
        Frontend will receive initial data: { server_info, tool_summaries }
        """
        server_parameters = value.get("server_parameters")
        mcp_client = MCPClient(server_parameters)

        tool_summaries = self.get_tool_summaries(mcp_client.get_tools())
        return {
            "server_parameters": server_parameters,
            # "server_info": server_info,
            "tool_summaries": tool_summaries
        }

    def example_payload(self):
        return {
            "action": "call_tool",
            "tool_name": "example_tool",
            "inputs": {"param": "value"},
            "server_parameters": {"url": "http://huggingface.co/mcp/", "transport": "streamable-http"},
        }

    def example_value(self):
        return {
            "server_parameters": {"url": "http://huggingface.co/mcp/", "transport": "streamable-http"},
            "server_info": {},
            "tool_summaries": []
        }

    def api_info(self):
        return {"type": {}, "description": "any valid json"}

    def example_input_for_param(self, param_schema):
            param_type = param_schema.get("type", "string")
            if param_type == "string":
                return "example text"
            elif param_type == "integer":
                return 42
            elif param_type == "number":
                return 3.14
            elif param_type == "boolean":
                return True
            elif param_type == "array":
                return []
            elif param_type == "object":
                return {}
            else:
                return None

    def get_tool_summaries(self, tools):
            tool_summaries = []
            for tool in tools:
                example_inputs = {
                    param_name: self.example_input_for_param(param_schema)
                    for param_name, param_schema in tool.inputs.items()
                }
                tool_summary = {
                    "name": tool.name,
                    "description": tool.description,
                    "inputs": tool.inputs,
                    "example_inputs": example_inputs,
                    "output_type": tool.output_type,
                }
                tool_summaries.append(tool_summary)
            return tool_summaries